import junit.runner.BaseTestRunner;
import org.junit.*;
import org.openqa.selenium.*;
import org.openqa.selenium.remote.DesiredCapabilities;
import org.openqa.selenium.remote.RemoteWebDriver;
import org.openqa.selenium.support.ui.ExpectedConditions;
import org.openqa.selenium.support.ui.WebDriverWait;

import java.net.URL;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

public class ManuscriptFlow {
    private static WebDriver driver = null;
    private static WebDriverWait wait = null;
    public String URL = Constants.URL;
    public static String manuscriptId;

    @BeforeClass
    public static void setUpClass() throws Exception {
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }

    @Before
    public void setUp() throws Exception {
        //driver = new RemoteWebDriver(new URL("http://localhost:4444/wd/hub"), DesiredCapabilities.firefox());
        driver = new RemoteWebDriver(new URL("http://localhost:4444/wd/hub"), DesiredCapabilities.chrome());
        //driver.manage().timeouts().implicitlyWait(5, TimeUnit.SECONDS);
        wait = new WebDriverWait(driver, 20);
        String window = driver.getWindowHandle();
        ((JavascriptExecutor) driver).executeScript("alert('Test')");
        driver.switchTo().alert().accept();
        driver.switchTo().window(window);
    }

    @After
    public void tearDown() throws Exception {
        driver.quit();
    }

    @Test
    public void createNewManuscript() throws Exception {
        /* Add constants.random after testing this function separably */
        Utils.validLogin( driver,  wait,  URL, Constants.email + /*Constants.randomStr*/ "auth1" + "@thinslices.com", Constants.password);

        WebElement validLogin = null;

        try {
            validLogin = wait.until(ExpectedConditions.presenceOfElementLocated(By.cssSelector("button[data-test='new-manuscript']")));
        } catch (NoSuchElementException e) {

        }

        assertNotNull(validLogin);

        driver.findElement(By.cssSelector("button[data-test='new-manuscript']")).click();

        try {
            wait.until(ExpectedConditions.presenceOfElementLocated(By.cssSelector("button[data-test='submission-next']")));
        } catch (NoSuchElementException e) {

        }

        assertNotNull(driver.findElement(By.cssSelector("button[data-test='submission-next']")));

        // First step submission flow
        driver.findElement(By.cssSelector("[data-test-id='agree-checkbox']")).click();
        driver.findElement(By.cssSelector("button[data-test='submission-next']")).click();

        // Second step submission flow
        try {
             wait.until(ExpectedConditions.presenceOfElementLocated(By.cssSelector(".wizard-step >[data-test-id='row']:nth-of-type(2) span")));
        } catch (NoSuchElementException e) {

        }

        assertEquals("Please provide the details of all the authors of this manuscript, in the order that they appear on the manuscript. Your details have been prefilled as the submitting author.",driver.findElement(By.cssSelector(".wizard-step >[data-test-id='row']:nth-of-type(2) span")).getText());
        assertEquals("MANUSCRIPT TITLE\n*", driver.findElement(By.cssSelector("[data-test-id='submission-title']")).getText());

        driver.findElement(By.cssSelector("input[name='metadata.title']")).sendKeys(Constants.manusName);
        driver.findElement(By.cssSelector("[data-test-id='submission-type']")).click();
        try {
            wait.until(ExpectedConditions.presenceOfElementLocated(By.cssSelector("[data-test-id='submission-type'] div[role=\"option\"]")));
        } catch (NoSuchElementException e) {

        }
        driver.findElement(By.cssSelector("[data-test-id='submission-type'] div[role=\"option\"]:nth-child(1)")).click();
        driver.findElement(By.cssSelector("[data-test-id='submission-abstract'] textarea")).sendKeys(Constants.manusAbstract);
        WebElement element = driver.findElement(By.cssSelector("[name='conflicts.hasConflicts'][value='no']"));

        Utils.scrollToElement(driver, element);
        element.click();

        try {
            wait.until(ExpectedConditions.presenceOfElementLocated(By.cssSelector("[name='conflicts.hasDataAvailability'][value='yes']")));
        } catch (NoSuchElementException e) {

        }

        driver.findElement(By.cssSelector("[name='conflicts.hasDataAvailability'][value='yes']")).click();
        WebElement element1 = driver.findElement(By.cssSelector("[name='conflicts.hasFunding'][value='yes']"));

        Utils.scrollToElement(driver, element1);
        element1.click();

        try {
            wait.until(ExpectedConditions.visibilityOfElementLocated(By.cssSelector("button[data-test='submission-next']")));
        } catch (NoSuchElementException e) {

        }

        WebElement element2 = driver.findElement(By.cssSelector("button[data-test='submission-next']"));

        Utils.scrollToElement(driver, element2);
        element2.click();

        // Third step submission flow

        try {
            wait.until(ExpectedConditions.visibilityOf(driver.findElement(By.cssSelector(".wizard-step [level='2']"))));
        } catch (NoSuchElementException e) {

        }
        WebElement pageTitle = driver.findElement(By.cssSelector(".wizard-step [level='2']"));
        assertEquals("3. Manuscript Files Upload", pageTitle.getText());

        try {
            wait.until(ExpectedConditions.visibilityOf(pageTitle));
        } catch (NoSuchElementException e) {

        }

        String mURL = driver.getCurrentUrl();
        manuscriptId = Utils.mID(mURL);

        driver.findElement(By.cssSelector(/*"[data-test='upload-manuscripts']*/"input[type=\"file\"]")).sendKeys(Constants.fileManuscript);
//        driver.findElement(By.cssSelector("[data-test='upload-supplementary'] input[type=\"file\"]")).sendKeys(Constants.fileSupplementary);
//        driver.findElement(By.cssSelector("[data-test='upload-coverLetter'] input[type=\"file\"]")).sendKeys(Constants.fileCoverLetter);


        try {
            wait.until(ExpectedConditions.visibilityOfElementLocated(By.cssSelector("button[data-test='submission-next']")));
        } catch (NoSuchElementException e) {

        }

        // Confirmation modal
        WebElement element3 = driver.findElement(By.cssSelector("button[data-test='submission-next']"));

        try {
            wait.until(ExpectedConditions.visibilityOf(element3));
        } catch (NoSuchElementException e) {

        }

        Utils.scrollToElement(driver, element3);
        element3.click();

        try {
            wait.until(ExpectedConditions.visibilityOfElementLocated(By.xpath("//button[contains(.,'AGREE & SUBMIT')]")));
        } catch (NoSuchElementException e) {

        }

        driver.findElement(By.xpath("//button[contains(.,'AGREE & SUBMIT')]")).click();

        try {
            wait.until(ExpectedConditions.visibilityOfElementLocated(By.xpath("//button[contains(.,'GO TO DASHBOARD')]")));
        } catch (NoSuchElementException e) {

        }

        driver.findElement(By.xpath("//button[contains(.,'GO TO DASHBOARD')]")).click();

        try {
            wait.until(ExpectedConditions.visibilityOfElementLocated(By.cssSelector("[data-test-id='fragment-" + manuscriptId + "']")));
        } catch (NoSuchElementException e) {

        }

        // Check if manuscript is displayed on dashboard
        assertNotNull("fragment-" + manuscriptId);

        driver.findElement(By.cssSelector("[data-test-id='fragment-" + manuscriptId + "']")).click();

        try {
            wait.until(ExpectedConditions.visibilityOfElementLocated(By.cssSelector("[data-test-id='fragment-status']")));
        } catch (NoSuchElementException e) {

        }

        // Check if manuscript status is submitted for author
        assertEquals("Submitted", driver.findElement(By.cssSelector("[data-test-id='fragment-status']")).getText());

    }

    @Test
    public void eqsApproveManuscript() throws Exception{
        String manuscriptDT = "[data-test-id='fragment-" + manuscriptId + "']";
        Utils.validLogin(driver, wait, URL, Constants.adminEmail, Constants.password);

        try {
            wait.until(ExpectedConditions.visibilityOfElementLocated(By.cssSelector("[data-test-id=\"row\"] button[type=\"button\"]")));
        } catch (NoSuchElementException e) {

        }

        driver.findElement(By.cssSelector("[data-test-id=\"row\"] button[type=\"button\"]")).click();
        driver.findElement(By.cssSelector("[data-test-id=\"row\"] div[role=\"option\"]:nth-child(1)")).click();

        driver.findElement(By.cssSelector("[data-test-id=\"item\"]:nth-of-type(2)")).click();
        driver.findElement(By.cssSelector("[data-test-id=\"row\"] div[role=\"option\"]:nth-child(1)")).click();

        driver.findElement(By.cssSelector(manuscriptDT)).click();

        try {
            wait.until(ExpectedConditions.presenceOfElementLocated(By.cssSelector("[data-test-id='button-qa-manuscript-" + manuscriptId + "']")));
        } catch (NoSuchElementException e) {

        }

        assertEquals("QA", driver.findElement(By.cssSelector("[data-test-id='fragment-status']")).getText());

        driver.findElement(By.cssSelector("[data-test-id='button-qa-manuscript-" + manuscriptId + "']")).click();

        try {
            wait.until(ExpectedConditions.visibilityOfElementLocated(By.xpath("//button[contains(.,'YES')]")));
        } catch (NoSuchElementException e) {

        }

        assertTrue(driver.getCurrentUrl().contains("eqs-decision"));

        driver.findElement(By.xpath("//button[contains(.,'YES')]")).click();

        try {
            wait.until(ExpectedConditions.visibilityOfElementLocated(By.cssSelector("input[name='customId'")));
        } catch (NoSuchElementException e) {

        }

        driver.findElement(By.cssSelector("input[name='customId'")).sendKeys(Constants.customManuscriptID);
        driver.findElement(By.xpath("//button[contains(.,'OK')]")).click();
    }
}
